#include <QBoxLayout>
#include <QPixmap>
#include <QIcon>
#include <QLabel>
#include <QTextEdit>
#include <QStyleOption>
#include <QPainter>
#include <QTextDocument>
#include <QDesktopServices>
#include "kaboutdialog.h"

namespace kdk
{
class KAboutDialogPrivate:public QObject
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KAboutDialog)

public:
    KAboutDialogPrivate(KAboutDialog*parent);
    void adjustMiniMode();
    void adjustNormalMode();

private:
    KAboutDialog *q_ptr;
    QLabel *m_pAppIconLabel;
    QLabel *m_pAppNameLabel;
    QLabel *m_pAppVersionLabel;
    QTextEdit *m_pBodyTextEdit;
    QLabel *m_pAppSupportLabel;
};

KAboutDialog::KAboutDialog(QWidget*parent,const QIcon& appIcon,const QString& appName,const QString& appVersion,const QString& appInfo)
    :KDialog(parent),
      d_ptr(new KAboutDialogPrivate(this))
{
    Q_D(KAboutDialog);
    setFixedWidth(420);
    setWindowIcon(appIcon);
    d->m_pAppIconLabel = new QLabel(this);
    QPixmap pix(appIcon.pixmap(QSize(96,96)));
    d->m_pAppIconLabel->setPixmap(pix);
    d->m_pAppIconLabel->setAlignment(Qt::AlignCenter);

    d->m_pAppNameLabel = new QLabel(this);
    d->m_pAppNameLabel->setText(appName);
    d->m_pAppNameLabel->setAlignment(Qt::AlignCenter);

    d->m_pAppVersionLabel = new QLabel(this);
    d->m_pAppVersionLabel->setText(appVersion);
    d->m_pAppVersionLabel->setAlignment(Qt::AlignCenter);

    d->m_pBodyTextEdit = new QTextEdit(this);
    d->m_pBodyTextEdit->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    d->m_pBodyTextEdit->setReadOnly(true);
    d->m_pBodyTextEdit->setText(appInfo);
    d->m_pBodyTextEdit->setTextInteractionFlags(Qt::NoTextInteraction);

    d->m_pAppSupportLabel = new QLabel(this);

    if(appInfo.isEmpty())
    {
        d->m_pBodyTextEdit->setVisible(false);
        d->adjustMiniMode();
    }
    else
    {
        d->m_pBodyTextEdit->setVisible(true);
        d->adjustNormalMode();
    }
    connect(d->m_pAppSupportLabel,&QLabel::linkActivated,this,[=](const QString url){
        QDesktopServices::openUrl(QUrl(url));
    });
    setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    changeTheme();
    connect(m_gsetting,&QGSettings::changed,this,[=](){changeTheme();});
}

KAboutDialog::~KAboutDialog()
{

}

void KAboutDialog::setAppIcon(const QIcon& icon)
{
    Q_D(KAboutDialog);
    QPixmap pix(icon.pixmap(QSize(96,96)));
    d->m_pAppIconLabel->setPixmap(pix);
    d->m_pAppIconLabel->setAlignment(Qt::AlignCenter);
}


void KAboutDialog::setAppName(const QString& appName)
{
    Q_D(KAboutDialog);
    d->m_pAppNameLabel->setText(appName);
}

QString KAboutDialog::appName()
{
    Q_D(KAboutDialog);
    return d->m_pAppNameLabel->text();
}

void KAboutDialog::setAppVersion(const QString& appVersion)
{
    Q_D(KAboutDialog);
    d->m_pAppVersionLabel->setText(appVersion);
}

QString KAboutDialog::appVersion()
{
    Q_D(KAboutDialog);
    return d->m_pAppVersionLabel->text();
}

void KAboutDialog::setBodyText(const QString& bodyText)
{
    Q_D(KAboutDialog);
    d->m_pBodyTextEdit->setText(bodyText);
}

QString KAboutDialog::bodyText()
{
    Q_D(KAboutDialog);
    return d->m_pBodyTextEdit->toPlainText();
}

void KAboutDialog::setAppSupport(const QString& appSupport)
{
    Q_D(KAboutDialog);
    d->m_pAppSupportLabel->setText(appSupport);
}

QString KAboutDialog::appSupport()
{
    Q_D(KAboutDialog);
    return d->m_pAppSupportLabel->text();
}

void KAboutDialog::setBodyTextVisiable(bool flag)
{
    Q_D(KAboutDialog);
    d->m_pBodyTextEdit->setVisible(flag);
    flag ? d->adjustNormalMode() : d->adjustMiniMode();
    update();
}

void KAboutDialog::changeTheme()
{
    Q_D(KAboutDialog);
    KDialog::changeTheme();
    QFont font;
    font.setPixelSize(ThemeController::systemFontSize() * 1.7);
    font.setWeight(300);
    d->m_pAppNameLabel->setFont(font);
    if(ThemeController::themeMode() == LightTheme)
    {
       d->m_pAppSupportLabel->setStyleSheet("QLabel{color:#595959;}");
       d->m_pAppVersionLabel->setStyleSheet("QLabel{color:#595959;}");
       d->m_pBodyTextEdit->setStyleSheet("QTextEdit{font-weight:400;line-height:24px;background-color:transparent;border:none;\
                                    color:#595959;}");
       d->m_pAppNameLabel->setStyleSheet("QLabel{color:#262626;}");
       d->m_pAppSupportLabel->setText(tr("Service & Support: ") +
                                              "<a href=\"mailto://support@kylinos.cn\""
                                              "style=\"color:#595959\">"
                                              "support@kylinos.cn</a>");
    }
    else
    {
      d->m_pAppSupportLabel->setStyleSheet("QLabel{color:#D9D9D9;}");
      d->m_pAppVersionLabel->setStyleSheet("QLabel{color:#D9D9D9;}");
      d->m_pBodyTextEdit->setStyleSheet("QTextEdit{font-weight:400;line-height:24px;background-color:transparent;border:none;\
                                 color:#D9D9D9;}");
      d->m_pAppNameLabel->setStyleSheet("QLabel{color:#D9D9D9;}");
      d->m_pAppSupportLabel->setText(tr("Service & Support: ") +
                                             "<a href=\"mailto://support@kylinos.cn\""
                                             "style=\"color:#D9D9D9\">"
                                             "support@kylinos.cn</a>");
    }
}

void KAboutDialog::paintEvent(QPaintEvent *paintEvent)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

KAboutDialogPrivate::KAboutDialogPrivate(KAboutDialog *parent)
    :q_ptr(parent)
{
    setParent(parent);
}

void KAboutDialogPrivate::adjustMiniMode()
{
    Q_Q(KAboutDialog);
    if(q->mainWidget()->layout())
        delete q->mainWidget()->layout();
    q->setMaximumHeight(312);
    m_pAppSupportLabel->setAlignment(Qt::AlignCenter);
    QVBoxLayout *vBoxLayout = new QVBoxLayout();
    vBoxLayout->setSpacing(0);
    vBoxLayout->addSpacing(24);
    vBoxLayout->addWidget(m_pAppIconLabel);
    vBoxLayout->addSpacing(16);
    vBoxLayout->addWidget(m_pAppNameLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pAppVersionLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pAppSupportLabel);
    vBoxLayout->setContentsMargins(32,0,32,0);
    vBoxLayout->addSpacing(40);
    q->mainWidget()->setLayout(vBoxLayout);
}

void KAboutDialogPrivate::adjustNormalMode()
{
    Q_Q(KAboutDialog);
    if(q->mainWidget()->layout())
        delete q->mainWidget()->layout();
    q->setMaximumHeight(560);
    m_pAppSupportLabel->setAlignment(Qt::AlignLeft);
    QVBoxLayout *vBoxLayout = new QVBoxLayout();
    vBoxLayout->addSpacing(20);
    vBoxLayout->addWidget(m_pAppIconLabel);
    vBoxLayout->addSpacing(16);
    vBoxLayout->addWidget(m_pAppNameLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pAppVersionLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pBodyTextEdit);
    QTextDocument *document = m_pBodyTextEdit->document();
    document->adjustSize();
    if(document->size().height()+10 < 200)
        m_pBodyTextEdit->setFixedHeight(document->size().height()+10);
    else
        m_pBodyTextEdit->setFixedHeight(200);
    vBoxLayout->addSpacing(24);
    vBoxLayout->addStretch();
    vBoxLayout->addWidget(m_pAppSupportLabel);
    vBoxLayout->setContentsMargins(32,0,32,0);
    vBoxLayout->addSpacing(40);
    q->mainWidget()->setLayout(vBoxLayout);
}

}

#include "kaboutdialog.moc"
#include "moc_kaboutdialog.cpp"
