#include <QLabel>
#include <QPixmap>
#include <QPushButton>
#include <QPropertyAnimation>
#include <QBoxLayout>
#include <QAction>
#include <QEvent>
#include <QStringListModel>
#include <QCompleter>
#include <QAbstractItemView>
#include <QStyledItemDelegate>
#include <QDebug>
#include <QStyleOption>
#include "ksearchlineedit.h"
#include "themeController.h"
#include <QApplication>

namespace kdk
{
static const int iconMargin = 6;
static const int buttonMargin = 6;
static const int icontextSpacing = 5;

class KSearchLineEditPrivate:public QObject,public ThemeController
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KSearchLineEdit)

public:
    KSearchLineEditPrivate(KSearchLineEdit*parent);

public Q_SLOTS:
    void doAnimationFinished();
    void doEditingFinished();

protected:
    void changeTheme();
private:
    KSearchLineEdit* q_ptr;
    QHBoxLayout* m_pWidgetLayout;
    QWidget* m_pWidget;
    QLabel* m_pTextLabel;
    QLabel* m_pIconLabel;
    QPushButton* m_pCancelButton;
    QPropertyAnimation* m_pAnimation;
    bool m_isSearching;
    QStringListModel* m_pListModel;
    QStringList m_List;
    QCompleter* m_pCompleter;
    QAction* m_pClearAction;
    QString m_placeHolderText;
    Qt::Alignment m_placeHolderAlignment;
    bool m_isClearButtonEnabled;
    QRect m_holderRect;
    void init();
    void adjustHolderRect();
};

KSearchLineEdit::KSearchLineEdit(QWidget *parent)
    :QLineEdit(parent),
      d_ptr(new KSearchLineEditPrivate(this))
{
    Q_D(KSearchLineEdit);

    setFocusPolicy(Qt::ClickFocus);
    setContextMenuPolicy(Qt::NoContextMenu);
    d->init();
    installEventFilter(this);

    connect(d->m_pAnimation,&QPropertyAnimation::finished,d,&KSearchLineEditPrivate::doAnimationFinished);
    connect(d->m_gsetting,&QGSettings::changed,d,&KSearchLineEditPrivate::changeTheme);
    connect(this,&KSearchLineEdit::editingFinished,d,&KSearchLineEditPrivate::doEditingFinished);
    connect(this,&KSearchLineEdit::editingFinished,this,&KSearchLineEdit::clearFocus);
    d->m_pClearAction = new QAction;
    this->addAction(d->m_pClearAction, QLineEdit::TrailingPosition);
    connect(d->m_pClearAction,&QAction::triggered,this,[=]{ KSearchLineEdit::clear();});
    d->m_pClearAction->setVisible(false);
    connect(this,&KSearchLineEdit::textChanged,this,[=]{
        if(this->text().isEmpty())
            d->m_pClearAction->setVisible(false);
        else
        {
            if(isClearButtonEnabled())
                d->m_pClearAction->setVisible(true);
        }

    });
    d->changeTheme();
}

KSearchLineEdit::~KSearchLineEdit()
{

}

void KSearchLineEdit::setEnabled(bool enable)
{
    Q_D(KSearchLineEdit);
    QLineEdit::setEnabled(enable);
    d->changeTheme();
}

bool KSearchLineEdit::isEnabled()
{
    Q_D(KSearchLineEdit);
    return QLineEdit::isEnabled();
}

void KSearchLineEdit::setClearButtonEnabled(bool enable)
{
    Q_D(KSearchLineEdit);
    d->m_isClearButtonEnabled = enable;
}

bool KSearchLineEdit::isClearButtonEnabled() const
{
    Q_D(const KSearchLineEdit);
    return d->m_isClearButtonEnabled;
}

QString KSearchLineEdit::placeholderText() const
{
    Q_D(const KSearchLineEdit);
    return QLineEdit::placeholderText();
}

void KSearchLineEdit::setPlaceholderText(const QString &text)
{
    Q_D(const KSearchLineEdit);
    d->m_pTextLabel->setText(text);
    d->m_pTextLabel->adjustSize();
}

Qt::Alignment KSearchLineEdit::placeholderAlignment() const
{
    Q_D(const KSearchLineEdit);
    return d->m_placeHolderAlignment;
}

void KSearchLineEdit::setPlaceholderAlignment(Qt::Alignment flag)
{
    Q_D(KSearchLineEdit);
    d->m_placeHolderAlignment = flag;
    d->adjustHolderRect();
}

Qt::Alignment KSearchLineEdit::alignment() const
{
    return QLineEdit::alignment();
}

void KSearchLineEdit::setAlignment(Qt::Alignment flag)
{
    Q_D(KSearchLineEdit);
    QLineEdit::setAlignment(flag);
}

void KSearchLineEdit::clear()
{
    Q_D(KSearchLineEdit);
    QFont font;
    font=QApplication::font();
    QFontMetrics fm(font);
    int m_height = fm.height();
    if(this->text().isEmpty())
        return;
    QLineEdit::clear();
    if(this->hasFocus())
    {
        d->m_pTextLabel->setVisible(false);
    }
    else
    {
        setPlaceholderAlignment(Qt::AlignCenter);

        d->m_pTextLabel->adjustSize();
        d->m_pWidget->setFixedHeight(m_height);
        d->adjustHolderRect();

        d->m_pTextLabel->setVisible(true);
        d->m_pAnimation->setStartValue(QRect(iconMargin,(height()- d->m_pWidget->height())/2,d->m_pWidget->width(),d->m_pWidget->height()));
        d->m_pAnimation->setEndValue(d->m_holderRect);
        d->m_pAnimation->start();
    }
}

bool KSearchLineEdit::eventFilter(QObject *watched, QEvent *event)
{
    Q_D(KSearchLineEdit);

    QFont font;
    font=QApplication::font();
    QFontMetrics fm(font);
    int m_height = fm.height();
    if(watched == this)
    {
        if(event->type() == QEvent::FocusIn)
        {
            d->m_isSearching = true;
            if(!this->text().isEmpty())
            {
                d->m_pTextLabel->setVisible(false);
            }
            else
            {
                setPlaceholderAlignment(Qt::AlignLeft);
                d->m_pTextLabel->setVisible(false);
                d->m_pAnimation->setStartValue(d->m_holderRect);
                d->m_pAnimation->setEndValue(QRect(iconMargin,(height()-d->m_pWidget->height())/2,d->m_pWidget->width(),d->m_pWidget->height()));
                d->m_pAnimation->start();
            }
        }
        else if(event->type() == QEvent::FocusOut)
        {
            d->m_isSearching = false;
            if(!this->text().isEmpty())
            {
                setPlaceholderAlignment(Qt::AlignLeft);
                d->m_pTextLabel->setVisible(false);
            }
            else
            {
                setPlaceholderAlignment(Qt::AlignCenter);
                d->m_pTextLabel->adjustSize();
                d->m_pWidget->setFixedHeight(m_height);
                d->adjustHolderRect();

                d->m_pTextLabel->setVisible(true);
                d->m_pAnimation->setStartValue(QRect(iconMargin,(height()- d->m_pWidget->height())/2,d->m_pWidget->width(),d->m_pWidget->height()));
                d->m_pAnimation->setEndValue(d->m_holderRect);
                d->m_pAnimation->start();

            }
        }
    }
    return QLineEdit::eventFilter(watched,event);
}

void KSearchLineEdit::resizeEvent(QResizeEvent *event)
{
    Q_D(KSearchLineEdit);
    d->adjustHolderRect();
}

void KSearchLineEdit::setVisible(bool visible)
{
    Q_D(KSearchLineEdit);

    d->m_pTextLabel->adjustSize();
    d->adjustHolderRect();
    d->m_pWidget->adjustSize();
    if(!text().isEmpty())
    {
        setPlaceholderAlignment(Qt::AlignLeft);
        d->m_pTextLabel->setVisible(false);
        d->m_pAnimation->setStartValue(d->m_holderRect);
        d->m_pAnimation->setEndValue(QRect(iconMargin,(height()-d->m_pWidget->height())/2,d->m_pWidget->width(),d->m_pWidget->height()));
        d->m_pAnimation->start();
    }
    else
    {
        setPlaceholderAlignment(Qt::AlignCenter);
        d->m_pTextLabel->setVisible(true);
    }

    QLineEdit::setVisible(visible);
}

KSearchLineEditPrivate::KSearchLineEditPrivate(KSearchLineEdit *parent)
    :q_ptr(parent),
      m_placeHolderText(tr("search")),
      m_placeHolderAlignment(Qt::AlignCenter),
      m_isClearButtonEnabled(true)
{
    Q_Q(KSearchLineEdit);
    m_pCompleter = new QCompleter(this);

    m_pCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    m_pListModel = new QStringListModel(m_List, this);
    m_pCompleter->setModel(m_pListModel);

    q->setCompleter(m_pCompleter);
    setParent(parent);
}

void KSearchLineEditPrivate::doAnimationFinished()
{
    Q_Q(KSearchLineEdit);
    if(m_isSearching)
        q->setTextMargins(iconMargin + m_pIconLabel->width(),0,0,0);
}

void KSearchLineEditPrivate::doEditingFinished()
{
    Q_Q(KSearchLineEdit);
    QString text = q->text();
    if(QString::compare(text,"") != 0)
    {
        bool flag = m_List.contains(text, Qt::CaseInsensitive);
         if(!flag)
         {
             m_List.append(text);
             m_pListModel->setStringList(m_List);
         }
    }
}

void KSearchLineEditPrivate::changeTheme()
{
    Q_Q(KSearchLineEdit);
    QPixmap pixmap = QIcon::fromTheme("search-symbolic").pixmap(QSize(16,16));
    initThemeStyle();
    if(!q->isEnabled())
    {
        QPalette palette = q->palette();
        palette.setColor(QPalette::Text,q->palette().color(QPalette::Disabled,QPalette::ButtonText));
        m_pTextLabel->setPalette(palette);
    }
    else
    {
        QPalette palette = q->palette();
        #if (QT_VERSION >= QT_VERSION_CHECK(5, 12, 0))
            palette.setColor(QPalette::Text,q->palette().color(QPalette::PlaceholderText));
        #else if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
            palette.setColor(QPalette::Text,q->palette().color(QPalette::Shadow));
        #endif

        m_pTextLabel->setPalette(palette);
    }
    if(ThemeController::themeMode() == LightTheme)
    {

        m_pIconLabel->setPixmap(pixmap);
        m_pClearAction->setIcon(QIcon::fromTheme("window-close-symbolic"));
    }
    else
    {
        m_pIconLabel->setPixmap(drawSymbolicColoredPixmap(pixmap));
        QPixmap pixmap = QIcon::fromTheme("window-close-symbolic").pixmap(QSize(16,16));
        m_pClearAction->setIcon(drawSymbolicColoredPixmap(pixmap));
    }
}

void KSearchLineEditPrivate::init()
{
    Q_Q(KSearchLineEdit);
    m_isSearching = false;
    m_pTextLabel = new QLabel(tr("Search"));
    m_pTextLabel->adjustSize();
    m_pIconLabel = new QLabel;
    QPixmap pixmap = QIcon::fromTheme("search-symbolic").pixmap(QSize(16,16));

    m_pIconLabel->setPixmap(pixmap);
    m_pIconLabel->setFixedSize(pixmap.size());
    m_pCancelButton = new QPushButton(QIcon::fromTheme("application-exit-symbolic"),"",q);
    auto size = m_pCancelButton->iconSize();
    m_pCancelButton->setFixedSize(size);
    m_pCancelButton->setCursor(Qt::ArrowCursor);
    m_pWidget =new QWidget(q);
    m_pWidget->setFocusPolicy(Qt::NoFocus);
    m_pWidgetLayout = new QHBoxLayout();
    m_pWidgetLayout->setContentsMargins(0,0,0,0);
    m_pWidgetLayout->setSpacing(icontextSpacing);
    m_pWidgetLayout->addWidget(m_pIconLabel,Qt::AlignVCenter);
    m_pWidgetLayout->addWidget(m_pTextLabel,Qt::AlignVCenter);
    m_pWidgetLayout->addStretch();
    m_pWidget->setLayout(m_pWidgetLayout);

    adjustHolderRect();
    m_pWidget->adjustSize();
    m_pCancelButton->setGeometry(QRect(q->width()-m_pCancelButton->width()-buttonMargin,(q->height()-m_pCancelButton->height())/2,m_pCancelButton->width(),m_pCancelButton->height()));
    m_pCancelButton->setVisible(false);

    m_pAnimation = new QPropertyAnimation(m_pWidget,"geometry");
    m_pAnimation->setEasingCurve(QEasingCurve::InOutQuad);
    m_pAnimation->setDuration(100);
    connect(m_gsetting,&QGSettings::changed,this,[=](const QString &key){
        if(key.contains("systemFontSize"))
        {
            m_pWidget->adjustSize();
            QString text = q->text();
            if(!(text.isEmpty()))
                q->setPlaceholderAlignment(Qt::AlignLeft);
            else
                q->setPlaceholderAlignment(Qt::AlignCenter);
            adjustHolderRect();
        }
    });
}

void KSearchLineEditPrivate::adjustHolderRect()
{
    Q_Q(KSearchLineEdit);
    if(m_placeHolderAlignment & Qt::AlignCenter)
    {
        m_holderRect = QRect((q->width()-(m_pIconLabel->width()+m_pTextLabel->width()+icontextSpacing))/2,
                            (q->height()-m_pWidget->height())/2,
                            m_pIconLabel->width()+m_pTextLabel->width()+icontextSpacing,
                            m_pWidget->height());

    }
    else if(m_placeHolderAlignment & Qt::AlignLeft)
    {
        m_holderRect = QRect(iconMargin,
                            (q->height()-m_pWidget->height())/2,
                            m_pIconLabel->width()+m_pTextLabel->width()+icontextSpacing,
                            m_pWidget->height());
    }

    else if(m_placeHolderAlignment & Qt::AlignRight)
    {
        m_holderRect = QRect(q->width()-(m_pIconLabel->width()+m_pTextLabel->width()+icontextSpacing),
                            (q->height()-m_pWidget->height())/2,
                            m_pIconLabel->width()+m_pTextLabel->width()+icontextSpacing,
                            m_pWidget->height());
    }
    m_pWidget->setGeometry(m_holderRect);
}

}

#include "ksearchlineedit.moc"
#include "moc_ksearchlineedit.cpp"
