#include "kslider.h"
#include "themeController.h"
#include <QPainter>
#include <QStyle>
#include <QStyleOptionSlider>
#include <QStyleOptionButton>
#include <QPalette>
#include <QMouseEvent>
#include <QDebug>

using namespace kdk;

namespace kdk {

static const int node_radius = 5;
static const int handle_radius = 8;
static const int spaceing = handle_radius+2;
static const int line_width = 4;
static const int line_border_radius = 2;

class KSliderPrivate :public QObject,public ThemeController
{
    Q_DECLARE_PUBLIC(KSlider)
public:
    KSliderPrivate(KSlider*parent);
    ~KSliderPrivate();

private:
    KSlider* q_ptr;
    KSliderType m_sliderType;
    void drawBaseNode(QPainter*painter);
    void drawCoverNode(QPainter*painter);
    void drawSlider(QPainter*painter);
    void locateNode();
    void drawBaseLine(QPainter*painter);
    void drawCoverLine(QPainter*painter);
    void updateValuePosition();
    void updateValue();
    void updateColor();
    void changeTheme();
    QRect baseLineRect();
    QRect coverLineRect();
    QList<QPoint>m_nodes;
    bool m_isPressed;
    bool m_isHovered;
    int m_valuePosition;
    QPoint m_currentpos;
    QColor m_baseColor;
    QColor m_coverColor;
    QColor m_startColor;
    QColor m_endColor;
    bool m_nodeVisible;
    QLinearGradient m_linearGradient;
};

KSliderPrivate::KSliderPrivate(KSlider *parent)
    :q_ptr(parent),
      m_sliderType(SmoothSlider),
      m_isPressed(false),
      m_isHovered(false),
      m_valuePosition(0),
      m_currentpos(QPoint(0,0)),
      m_nodeVisible(true)
{
}

KSliderPrivate::~KSliderPrivate()
{

}

void KSliderPrivate::drawBaseNode(QPainter*painter)
{
    Q_Q(KSlider);
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setBrush(m_baseColor);
    for(auto point : m_nodes)
    {
        painter->drawEllipse(point,node_radius,node_radius);
    }
}

void KSliderPrivate::drawCoverNode(QPainter*painter)
{
    Q_Q(KSlider);
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(Qt::NoPen);
    painter->setBrush(q->palette().color(QPalette::Highlight));

    if(q->orientation() == Qt::Horizontal)
    {
        for(auto point : m_nodes)
        {
            if(point.x()<= m_valuePosition)
                painter->drawEllipse(point,node_radius,node_radius);
        }
    }
    else
    {
        for(auto point : m_nodes)
        {
            if(point.y()>= m_valuePosition)
                painter->drawEllipse(point,node_radius,node_radius);
        }
    }
}

void KSliderPrivate::drawSlider(QPainter*painter)
{
    Q_Q(KSlider);
    QStyleOptionSlider sliderOption;
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(Qt::NoPen);
    //填充颜色待定
    if(ThemeController::widgetTheme() == FashionTheme && m_isHovered&&!m_isPressed)
    {
        painter->setBrush(m_linearGradient);
    }
    else
        painter->setBrush(m_startColor);
    //获取滑动块大小
    QRect handle = q->style()->proxy()->subControlRect(QStyle::CC_Slider, &sliderOption, QStyle::SC_SliderHandle, q);
    int handle_radius;
    QPoint point;
    if(q->orientation() == Qt::Horizontal)
    {
        handle_radius = handle.width()/2;
        point = QPoint(m_valuePosition,q->height()/2);
    }
    else
    {
        handle_radius = handle.height()/2;
        point = QPoint(q->width()/2,m_valuePosition);
    }
    painter->drawEllipse(point,handle_radius,handle_radius);
}

void KSliderPrivate::locateNode()
{
    Q_Q(KSlider);
    if(q->orientation() == Qt::Horizontal)
    {
        m_nodes.clear();
        QPoint beginNode(baseLineRect().left(),q->height()/2);
        QPoint endNode(baseLineRect().right(),q->height()/2);
        m_nodes.append(beginNode);

        if(!q->tickInterval())
        {
            m_nodes.append(endNode);
            return;
        }

        for(int i = q->minimum() + q->tickInterval();i < q->maximum();i += q->tickInterval())
        {
            QPoint point;
            int x = baseLineRect().left()+baseLineRect().width()*(i-q->minimum())/(q->maximum()-q->minimum());
            int y = q->height()/2;
            point.setX(x);
            point.setY(y);
            m_nodes.append(point);
        }
        m_nodes.append(endNode);
    }
    else
    {
        m_nodes.clear();
        QPoint beginNode(q->width()/2,baseLineRect().bottom());
        QPoint endNode(q->width()/2,baseLineRect().top());
        m_nodes.append(beginNode);
        if(!q->tickInterval())
        {
            m_nodes.append(endNode);
            return;
        }
        for(int i = q->minimum() + q->tickInterval();i < q->maximum();i += q->tickInterval())
        {
            QPoint point;
            int x = q->width()/2;
            int y = baseLineRect().top() + baseLineRect().height()*(i-q->minimum())/(q->maximum()-q->minimum());
            point.setX(x);
            point.setY(y);
            m_nodes.append(point);
        }
        m_nodes.append(endNode);
    }
}

void KSliderPrivate::drawBaseLine(QPainter*painter)
{
    Q_Q(KSlider);
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(Qt::NoPen);
    painter->setBrush(m_baseColor);
    painter->drawRoundedRect(this->baseLineRect(),line_border_radius,line_border_radius);
}

void KSliderPrivate::drawCoverLine(QPainter*painter)
{
    Q_Q(KSlider);
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(Qt::NoPen);
    painter->setBrush(m_coverColor);
    painter->drawRoundedRect(this->coverLineRect(),line_border_radius,line_border_radius);
}

void KSliderPrivate::updateValuePosition()
{
    Q_Q(KSlider);
    if(q->orientation() == Qt::Horizontal)
    {
        m_valuePosition =this->baseLineRect().left() +
                this->baseLineRect().width()*(q->value()-q->minimum())/(q->maximum()-q->minimum());
    }
    else
    {
        m_valuePosition =this->baseLineRect().top()+this->baseLineRect().height()
                -this->baseLineRect().height()*(q->value()-q->minimum())/(q->maximum()-q->minimum());
    }

}

void KSliderPrivate::updateValue()
{
    Q_Q(KSlider);
    int dur = q->maximum() - q->minimum();
    int pos;
    int step = q->singleStep();
    int nodeInterval =q->tickInterval();
    if(q->orientation() == Qt::Horizontal)
    {
        if(m_currentpos.x() > baseLineRect().right())
            q->setValue(q->maximum());
        else if(m_currentpos.x() < baseLineRect().left())
            q->setValue(q->minimum());
        pos = qRound(q->minimum()+dur*((double)m_currentpos.x()-baseLineRect().left())/baseLineRect().width());
        int left = baseLineRect().left();
        int width = baseLineRect().width();
        switch (m_sliderType)
        {
        case SingleSelectSlider:
        case SmoothSlider:
            q->setValue(pos);
            break;
        case StepSlider:
        {
            int frontIndex = (pos-q->minimum()) / step;
            int backIndex = (pos-q->minimum()) / step + 1;
            if((m_currentpos.x()-left-frontIndex*step*width/dur)
                    <(backIndex*step*width/dur - (m_currentpos.x()-left)))
                pos = frontIndex * step + q->minimum();
            else
                pos = backIndex * step + q->minimum();
            q->setValue(pos);
        }
            break;
        case NodeSlider:
        {
            int frontIndex = (pos-q->minimum()) / nodeInterval;
            int backIndex = (pos-q->minimum()) / nodeInterval + 1;
            if((m_currentpos.x()-left-frontIndex*nodeInterval*width/dur)
                    <(backIndex*nodeInterval*width/dur - (m_currentpos.x()-left)))
                pos = frontIndex * nodeInterval + q->minimum();
            else
                pos = backIndex * nodeInterval + q->minimum();
            q->setValue(pos);
        }
            break;
        default:
            break;
        }
    }
    else
    {
        if(m_currentpos.y() < baseLineRect().top())
            q->setValue(q->maximum());
        else if(m_currentpos.y() > baseLineRect().bottom())
            q->setValue(q->minimum());
        pos = qRound(q->minimum()+dur*((double)q->height()-m_currentpos.y())/q->height());
        int top = baseLineRect().top();
        int height = baseLineRect().height();
        switch (m_sliderType)
        {
        case SingleSelectSlider:
        case SmoothSlider:
            q->setValue(pos);
            break;
        case StepSlider:
        {
            int frontIndex = (pos-q->minimum())  / step + 1;
            int backIndex = (pos-q->minimum())  / step ;
            if((m_currentpos.y()-top - (height-frontIndex*step*height/dur))<
                    ((height-backIndex*step*height/dur) - (m_currentpos.y()-top)))
                pos = frontIndex * step + q->minimum();
            else
                pos = backIndex * step + q->minimum();
            q->setValue(pos);
        }
            break;
        case NodeSlider:
        {
            int frontIndex = (pos-q->minimum()) / nodeInterval + 1;
            int backIndex = (pos-q->minimum()) / nodeInterval ;
            if((m_currentpos.y()-top - (height-frontIndex*nodeInterval*height/dur))<
                    ((height-backIndex*nodeInterval*height/dur) - (m_currentpos.y()-top)))
                pos = frontIndex * nodeInterval + q->minimum();
            else
                pos = backIndex * nodeInterval + q->minimum();
            q->setValue(pos);
        }
            break;
        default:
            break;
        }
    }
}

void KSliderPrivate::updateColor()
{
    Q_Q(KSlider);
    QColor highlightColor = q->palette().color(QPalette::Highlight);

    m_baseColor = q->palette().color(QPalette::Button);
    if(!q->isEnabled())
    {
        m_coverColor = q->palette().color(QPalette::Disabled,QPalette::ButtonText);
        m_startColor = q->palette().color(QPalette::Disabled,QPalette::ButtonText);
        return;
    }
    if(ThemeController::themeMode() == LightTheme)
    {
        QColor whiteColor("#FFFFFF");
        QColor darkColor("#000000");
        if(m_isPressed)
        {
            m_coverColor = highlightColor.darker(120).darker(120);
            m_startColor = highlightColor.darker(120);
            return;
        }
        else if(m_isHovered)
        {
            m_coverColor = highlightColor.darker(120);
            if(ThemeController::widgetTheme() == FashionTheme)
            {
                m_startColor = ThemeController::mixColor(QColor("#3E76F7"),whiteColor,0.2);
                m_endColor = ThemeController::mixColor(QColor("#3E76F7"),darkColor,0.05);

                m_linearGradient.setColorAt(0,m_startColor);
                m_linearGradient.setColorAt(1,m_endColor);
            }
            else
                m_startColor = highlightColor.darker(105);
            return;
        }
        else
        {
            m_coverColor = highlightColor.darker(120);
            m_startColor = highlightColor;
        }
    }
    else
    {
        QColor whiteColor("#FFFFFF");
        if(m_isPressed)
        {
            m_coverColor = highlightColor.lighter(120).lighter(120);
            m_startColor = highlightColor.lighter(120);
            return;
        }
        else if(m_isHovered)
        {
            m_coverColor = highlightColor.lighter(120);
            if(ThemeController::widgetTheme() == FashionTheme)
            {
                 m_startColor = ThemeController::mixColor(QColor("#3E76F7"),whiteColor,0.2);
                 m_endColor = QColor("#3E76F7");

                 m_linearGradient.setColorAt(0,m_startColor);
                 m_linearGradient.setColorAt(1,m_endColor);
            }
            else
                m_startColor = highlightColor.lighter(105);
            return;
        }
        else
        {
            m_coverColor = highlightColor.lighter(120);
            m_startColor = highlightColor;
        }
    }
}

void KSliderPrivate::changeTheme()
{
    Q_Q(KSlider);
    initThemeStyle();
    updateColor();
    q->update();
}

QRect KSliderPrivate::baseLineRect()
{
    Q_Q(KSlider);
    QRect rect;
    if(q->orientation() == Qt::Horizontal)
    {
        rect.setBottom(q->height()/2 + line_width/2);
        rect.setTop(q->height()/2 - line_width/2);
        rect.setLeft(spaceing);
        rect.setRight(q->width() - spaceing);
    }
    else
    {
        rect.setBottom(q->height() - spaceing);
        rect.setTop(spaceing);
        rect.setLeft(q->width()/2 - line_width/2);
        rect.setRight(q->width()/2 + line_width/2);
    }
    return rect;
}

QRect KSliderPrivate::coverLineRect()
{
    Q_Q(KSlider);
    QRect rect;
    if(q->orientation() == Qt::Horizontal)
    {
        rect.setBottom(q->height()/2 + line_width/2);
        rect.setTop(q->height()/2 - line_width/2);
        rect.setLeft(spaceing);
        rect.setRight(m_valuePosition);
    }
    else
    {
        rect.setBottom(q->height() - spaceing);
        rect.setTop(m_valuePosition);
        rect.setLeft(q->width()/2 - line_width/2);
        rect.setRight(q->width()/2 + line_width/2);
    }
    return rect;
}

KSlider::KSlider(QWidget *parent)
    :KSlider(Qt::Horizontal,parent)
{
}

KSlider::KSlider(Qt::Orientation orientation, QWidget *parent)
    :QSlider(orientation,parent),d_ptr(new KSliderPrivate(this))
{
    Q_D(KSlider);
    d->changeTheme();
    connect(d->m_gsetting,&QGSettings::changed, d,&KSliderPrivate::changeTheme);
    installEventFilter(this);
}

void KSlider::setTickInterval(int interval)
{
    QSlider::setTickInterval(interval);
    update();
}

void KSlider::setSliderType(KSliderType type)
{
    Q_D(KSlider);
    d->m_sliderType = type;
    if(!tickInterval() && type == KSliderType::NodeSlider)
        setTickInterval(10);
    if(!singleStep()  && type == KSliderType::StepSlider)
        setSingleStep(10);
    if(type == KSliderType::SingleSelectSlider)
    {
        setTickInterval(1);
        setSingleStep(1);
        setRange(0,2);
    }
}

KSliderType KSlider::sliderType()
{
    Q_D(KSlider);
    return d->m_sliderType;
}

int KSlider::tickInterval() const
{
    return QSlider::tickInterval();
}

void KSlider::setValue(int value)
{
    Q_D(KSlider);
    QSlider::setValue(value);
    update();
}

void KSlider::setNodeVisible(bool flag)
{
    Q_D(KSlider);
    if(d->m_nodeVisible != flag)
    {
        d->m_nodeVisible = flag;
        repaint();
    }
}

bool KSlider::nodeVisible()
{
    Q_D(KSlider);
    return d->m_nodeVisible;
}

void KSlider::paintEvent(QPaintEvent *event)
{
    Q_D(KSlider);

    if(orientation() == Qt::Horizontal)
    {
        for(auto point : d->m_nodes)
        {
            if(point.x()<= d->m_valuePosition)
                d->m_linearGradient=QLinearGradient(point.x(),point.y()-node_radius,point.x(),point.y()+node_radius);
        }
    }
    else
    {
        for(auto point : d->m_nodes)
        {
            if(point.y()>= d->m_valuePosition)
                d->m_linearGradient=QLinearGradient(point.x(),point.y()-node_radius,point.x(),point.y()+node_radius);
        }
    }

    d->updateColor();
    d->locateNode();
    d->updateValuePosition();
    QPainter p(this);
    d->drawBaseLine(&p);
    if(d->m_sliderType != SingleSelectSlider)
        d->drawCoverLine(&p);
    if(d->m_nodeVisible)
    {
        d->drawBaseNode(&p);
        if(d->m_sliderType != SingleSelectSlider)
            d->drawCoverNode(&p);
    }
    d->drawSlider(&p);
}

void KSlider::resizeEvent(QResizeEvent *event)
{
    QSlider::resizeEvent(event);
}

void KSlider::mousePressEvent(QMouseEvent *event)
{
    Q_D(KSlider);

    if(event->button() == Qt::LeftButton)
    {
        d->m_currentpos = event->pos();
        d->m_isPressed = true;
        d->updateColor();
        d->updateValue();
    }
}

void KSlider::mouseReleaseEvent(QMouseEvent *event)
{
    Q_D(KSlider);
    if(event->button() == Qt::LeftButton)
    {
        d->m_currentpos = event->pos();
        d->m_isPressed = false;
        d->updateColor();
        update();
    }
}

void KSlider::mouseMoveEvent(QMouseEvent *event)
{
    Q_D(KSlider);

    if(d->m_isPressed)
    {
        d->m_currentpos = event->pos();
        d->updateValue();
    }
}

void KSlider::wheelEvent(QWheelEvent *event)
{
    //to do
}

bool KSlider::eventFilter(QObject *watched, QEvent *event)
{
    Q_D(KSlider);
    if(watched == this)
    {
        switch (event->type()) {
        case QEvent::Enter:
            d->m_isHovered = true;
            d->updateColor();
            break;
        case QEvent::Leave:
            d->m_isHovered = false;
            d->updateColor();
        default:
            break;
        }
    }
    return QSlider::eventFilter(watched,event);
}
}
//#include "kslider.moc"
#include "moc_kslider.cpp"
