#include "kwidget.h"
#include "gui_g.h"
#include "xatom-helper.h"
#include <QBoxLayout>
#include <QPalette>
#include <QPainter>
#include <QApplication>

namespace kdk
{
class KWidgetPrivate:public QObject
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KWidget)

public:
    KWidgetPrivate(KWidget*parent);

private:
    KWidget *q_ptr;
    KIconBar* m_pIconBar;
    KWindowButtonBar* m_pWindowButtonBar;
    QHBoxLayout* m_pTitleLayout;
    QHBoxLayout* m_pWidgetLayout;
    QVBoxLayout* m_pMainLayout;
    QWidget* m_pBaseWidget;
    QWidget* m_pSideWidget;
    QString m_IconName;
    QGSettings* m_pTransparency;
    double tranSparency;
    LayoutType m_layoutType;
};

KWidget::KWidget(QWidget *parent)
    :QWidget(parent),
      d_ptr(new KWidgetPrivate(this))
{
    Q_D(KWidget);
    setFocusPolicy(Qt::ClickFocus);
    setObjectName("Kwidget");
    this->setAttribute(Qt::WA_TranslucentBackground, true);  //透明
    this->setProperty("useSystemStyleBlur", true);
    connect(d->m_pWindowButtonBar->minimumButton(),&QPushButton::clicked,this,&KWidget::showMinimized);
    connect(d->m_pWindowButtonBar->maximumButton(),&QPushButton::clicked,this,[=]()
        {
            if(this->isMaximized())
                showNormal();
            else
                showMaximized();
        });
    connect(d->m_pWindowButtonBar->closeButton(),&QPushButton::clicked,this,&KWidget::close);
    connect(d->m_pWindowButtonBar,&KWindowButtonBar::doubleClick,this,[=]()
        {
            if(this->isMaximized())
                showNormal();
            else
                showMaximized();
        });
    connect(d->m_pIconBar,&KIconBar::doubleClick,this,[=]()
        {
            if(this->isMaximized())
                showNormal();
            else
                showMaximized();
        });
    changeIconStyle();
    connect(m_gsetting,&QGSettings::changed,this,[=](){changeIconStyle();});

    if (QGSettings::isSchemaInstalled("org.ukui.control-center.personalise")) {
        d->m_pTransparency = new QGSettings("org.ukui.control-center.personalise");
        d->tranSparency = d->m_pTransparency->get("transparency").toDouble();
        connect(d->m_pTransparency, &QGSettings::changed, this, [=](QString value) {
            if (value == "transparency") {
                d->tranSparency = d->m_pTransparency->get("transparency").toDouble();
                changeTheme();
            }
        });
    }

    changeTheme();
    connect(m_gsetting,&QGSettings::changed,this,[=](){changeTheme();});


    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(this->winId(), hints);
    installEventFilter(this);
    resize(800,600);
}

KWidget::~KWidget()
{

}

void KWidget::setIcon(const QIcon &icon)
{
    Q_D(KWidget);
    d->m_IconName = icon.name();
    d->m_pIconBar->setIcon(icon);
    setWindowIcon(icon.pixmap(QSize(36,36)));
}

void KWidget::setIcon(const QString &iconName)
{
    Q_D(KWidget);
    d->m_IconName = iconName;
    d->m_pIconBar->setIcon(iconName);
    setWindowIcon(QIcon::fromTheme(iconName).pixmap(QSize(36,36)));
}

void KWidget::setWidgetName(const QString &widgetName)
{
    Q_D(KWidget);
    d->m_pIconBar->setWidgetName(widgetName);
}

QWidget *KWidget::sideBar()
{
    Q_D(KWidget);
    return d->m_pSideWidget;
}

QWidget *KWidget::baseBar()
{
    Q_D(KWidget);
    return d->m_pBaseWidget;
}

KWindowButtonBar *KWidget::windowButtonBar()
{
    Q_D(KWidget);
    return d->m_pWindowButtonBar;
}

KIconBar *KWidget::iconBar()
{
    Q_D(KWidget);
    return d->m_pIconBar;
}

void KWidget::setLayoutType(LayoutType type)
{
    Q_D(KWidget);
    d->m_layoutType = type;
    changeTheme();
}

bool KWidget::eventFilter(QObject *target, QEvent *event)
{
    Q_D(KWidget);
    if(target == this && event->type() == QEvent::WindowStateChange)
    {
        if(this->isMaximized())
            d->m_pWindowButtonBar->setMaximumButtonState(MaximumButtonState::Restore);
        else
            d->m_pWindowButtonBar->setMaximumButtonState(MaximumButtonState::Maximum);
    }
    return QWidget::eventFilter(target, event);
}

void KWidget::changeIconStyle()
{
    Q_D(KWidget);
    initThemeStyle();
    setIcon(d->m_IconName);
}

void KWidget::changeTheme()
{
    Q_D(KWidget);
    initThemeStyle();
    QColor baseColor = this->palette().color(QPalette::Base);
    QColor sideColor = this->palette().color(QPalette::Window);
    sideColor.setAlphaF(d->tranSparency);
    QPalette basePalette = this->palette();
    basePalette.setColor(QPalette::Window,baseColor);
    d->m_pWindowButtonBar->setAutoFillBackground(true);
    d->m_pWindowButtonBar->setPalette(basePalette);
    d->m_pBaseWidget->setAutoFillBackground(true);
    d->m_pBaseWidget->setPalette(basePalette);

    QPalette sidePalette = this->palette();
    switch (d->m_layoutType)
    {
    case VerticalType:
        sidePalette.setColor(QPalette::Window,sideColor);
        d->m_pIconBar->setAutoFillBackground(true);
        d->m_pIconBar->setPalette(basePalette);
        d->m_pSideWidget->hide();
        break;
    case HorizontalType:
        sidePalette.setColor(QPalette::Window,sideColor);
        d->m_pSideWidget->setAutoFillBackground(true);
        d->m_pSideWidget->setPalette(sidePalette);
        d->m_pIconBar->setAutoFillBackground(true);
        d->m_pIconBar->setPalette(sidePalette);
        d->m_pSideWidget->show();
        break;
    case MixedType:
        sidePalette.setColor(QPalette::Window,sideColor);
        d->m_pSideWidget->setAutoFillBackground(true);
        d->m_pSideWidget->setPalette(sidePalette);
        d->m_pSideWidget->show();
        d->m_pIconBar->setAutoFillBackground(true);
        d->m_pIconBar->setPalette(basePalette);
        break;
    default:
        break;
    }

}

KWidgetPrivate::KWidgetPrivate(KWidget *parent)
    :q_ptr(parent)
{
    Q_Q(KWidget);
    m_pMainLayout = new QVBoxLayout();
    m_pTitleLayout = new QHBoxLayout();
    m_layoutType = VerticalType;

    //图标和标题名称
    m_pIconBar = new KIconBar(q);
    m_pIconBar->setFixedWidth(200);
    m_pIconBar->setObjectName("IconBar");

    //窗口三联
    m_pWindowButtonBar = new KWindowButtonBar(q);
    m_pWindowButtonBar->setObjectName("TitleBar");
    m_pWindowButtonBar->setFixedHeight(40);

    m_pTitleLayout->addWidget(m_pIconBar);
    m_pTitleLayout->addWidget(m_pWindowButtonBar);

    m_pWidgetLayout = new QHBoxLayout();
    m_pBaseWidget = new QWidget(q);
    m_pBaseWidget->setObjectName("BaseWidget");
    m_pSideWidget = new QWidget(q);
    m_pSideWidget->setObjectName("SideWidget");
    m_pSideWidget->setFixedWidth(200);
    m_pWidgetLayout->addWidget(m_pSideWidget);
    m_pWidgetLayout->addWidget(m_pBaseWidget);

    m_pMainLayout->addLayout(m_pTitleLayout);
    m_pMainLayout->addLayout(m_pWidgetLayout);

    m_pMainLayout->setSpacing(0);
    m_pMainLayout->setContentsMargins(0,0,0,0);
    q->setLayout(m_pMainLayout);
    setParent(parent);
}
}

#include "kwidget.moc"
#include "moc_kwidget.cpp"
